@@ -2,12 +2,11 @@ package WebService::Google::Reader;
 
 use strict;
 use warnings;
-use parent qw(Class::Accessor::Fast);
 
 use Carp qw(croak);
 use HTTP::Request::Common qw(GET POST);
 use LWP::UserAgent;
-use JSON;
+use JSON::MaybeXS;
 use URI;
 use URI::Escape;
 use URI::QueryParam;
@@ -16,17 +15,17 @@ use WebService::Google::Reader::Constants;
 use WebService::Google::Reader::Feed;
 use WebService::Google::Reader::ListElement;
 
-our $VERSION = '0.22';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
-__PACKAGE__->mk_accessors(qw(
-    auth compress error host password response scheme token ua username
-));
+use Class::Tiny qw(
+    appid appkey auth compress error host password response _scheme token ua
+    username
+);
 
-sub new {
-    my ($class, %params) = @_;
 
-    my $self = bless { %params }, $class;
+sub BUILD {
+    my ($self, $params) = @_;
 
     croak q('host' is required) unless $self->host;
     my $host = $self->host;
@@ -34,28 +33,27 @@ sub new {
     $host =~ s[/+$][];
     $self->host($host);
 
-    my $ua = $params{ua};
-    unless (ref $ua and $ua->isa(q(LWP::UserAgent))) {
-        $ua = LWP::UserAgent->new(
-            agent => __PACKAGE__.'/'.$VERSION,
-        );
-        $self->ua($ua);
+    if ($host =~ /(^|\.)inoreader\.com$/i) {
+        croak q('appid' and 'appkey' are required by inoreader)
+            unless defined $self->appid and defined $self->appkey;
     }
 
-    $self->compress(1);
-    $self->debug(0);
-    for my $accessor (qw( compress debug )) {
-        $self->$accessor($params{$accessor}) if exists $params{$accessor};
+    unless (ref $self->ua and $self->ua->isa(q(LWP::UserAgent))) {
+       $self->ua(LWP::UserAgent->new(agent => __PACKAGE__.'/'.$VERSION));
     }
 
-    $ua->default_header(accept_encoding => 'gzip,deflate')
+    $self->compress(exists $params->{compress} ? $params->{compress} : 1);
+    $self->debug(exists $params->{debug}       ? $params->{debug}    : 0);
+
+    $self->ua->default_header(accept_encoding => 'gzip,deflate')
         if $self->compress;
 
-    $self->scheme($params{secure} || $params{https} ? 'https' : 'http');
+    $self->_scheme($params->{secure} || $params->{https} ? 'https' : 'http');
 
     return $self;
 }
 
+
 sub debug {
     my ($self, $val) = @_;
     return $self->{debug} unless 2 == @_;
@@ -74,6 +72,7 @@ sub debug {
     return $self->{debug} = $val;
 }
 
+
 ## Feeds
 
 sub feed  { shift->_feed(feed  => shift, @_) }
@@ -89,6 +88,7 @@ sub unread  {
     );
 }
 
+
 sub search {
     my ($self, $query, %params) = @_;
 
@@ -110,7 +110,7 @@ sub search {
     my $res = $self->_request($req) or return;
 
     my @ids = do {
-        my $ref = eval { from_json($res->decoded_content) } or do {
+        my $ref = eval { JSON->new->decode($res->decoded_content) } or do {
             $self->error("Failed to parse JSON response: $@");
             return;
         };
@@ -121,18 +121,17 @@ sub search {
         @ids = reverse @ids if 'asc' eq $order;
     }
 
-    my $feed = (__PACKAGE__.'::Feed')->new(
-        request => $req, ids => \@ids, count => $params{count} || 40,
-    );
+    my $feed = (__PACKAGE__.'::Feed')->new(_request => $req, _ids => \@ids);
     return $self->more($feed);
 }
 
+
 sub more {
     my ($self, $feed) = @_;
 
     my $req;
-    if (defined $feed->ids) {
-        my @ids = splice @{$feed->ids}, 0, $feed->count;
+    if (defined $feed->_ids) {
+        my @ids = splice @{$feed->_ids}, 0, $feed->count;
         return unless @ids;
 
         my $uri = URI->new($self->host . STREAM_ITEMS_CONTENTS_PATH);
@@ -141,12 +140,12 @@ sub more {
     elsif ($feed->elem) {
         return unless defined $feed->continuation;
         return if $feed->entries < $feed->count;
-        $req = $feed->request;
+        $req = $feed->_request;
         my $prev_continuation = $req->uri->query_param('c') || '';
         return if $feed->continuation eq $prev_continuation;
         $req->uri->query_param(c => $feed->continuation);
     }
-    elsif ($req = $feed->request) {
+    elsif ($req = $feed->_request) {
         # Initial request.
     }
     else { return }
@@ -159,6 +158,7 @@ sub more {
 
 *previous = *previous = *next = *next = \&more;
 
+
 ## Lists
 
 sub tags        { $_[0]->_list($_[0]->host . LIST_TAGS_PATH) }
@@ -179,6 +179,7 @@ sub delete_tag      { shift->edit_tag(_listify(\@_), delete => 1) }
 sub mark_read_tag   { shift->mark_read(tag   => _listify(\@_)) }
 sub mark_read_state { shift->mark_read(state => _listify(\@_)) }
 
+
 sub rename_feed_tag {
     my ($self, $old, $new) = @_;
 
@@ -203,6 +204,7 @@ sub rename_feed_tag {
     return $self->edit_feed(\@tagged, tag => $new, untag => $old);
 }
 
+
 sub rename_entry_tag {
     my ($self, $old, $new) = @_;
 
@@ -218,13 +220,15 @@ sub rename_entry_tag {
     return 1;
 }
 
+
 sub rename_tag {
     my $self = shift;
-    return unless $self->rename_tag_feed(@_);
-    return unless $self->rename_tag_entry(@_);
-    return $self->delete_tags(shift);
+    return unless $self->rename_feed_tag(@_);
+    return unless $self->rename_entry_tag(@_);
+    return $self->delete_tag(shift);
 }
 
+
 ## Edit feeds
 
 sub edit_feed {
@@ -283,6 +287,7 @@ sub edit_feed {
     return $self->_edit($url, %fields);
 }
 
+
 sub tag_feed       { shift->edit_feed(shift, tag     => \@_) }
 sub untag_feed     { shift->edit_feed(shift, untag   => \@_) }
 sub state_feed     { shift->edit_feed(shift, state   => \@_) }
@@ -292,6 +297,7 @@ sub unsubscribe    { shift->edit_feed(_listify(\@_), unsubscribe => 1) }
 sub mark_read_feed { shift->mark_read(feed => _listify(\@_)) }
 sub rename_feed    { $_[0]->edit_feed($_[1], title => $_[2]) }
 
+
 ## Edit entries
 
 sub edit_entry {
@@ -332,6 +338,7 @@ sub edit_entry {
     return $self->_edit($url, %fields);
 }
 
+
 sub tag_entry     { shift->edit_entry(shift, tag     => \@_) }
 sub untag_entry   { shift->edit_entry(shift, untag   => \@_) }
 sub state_entry   { shift->edit_entry(shift, state   => \@_) }
@@ -350,6 +357,7 @@ for (qw(star unstar like unlike)) {
     *$_ = \&{$_.'_entry'};
 }
 
+
 ## Miscellaneous
 
 sub mark_read {
@@ -367,6 +375,7 @@ sub mark_read {
     return $self->_edit($self->host . EDIT_MARK_READ_PATH, %fields);
 }
 
+
 sub edit_preference {
     my ($self, $key, $val) = @_;
 
@@ -376,6 +385,7 @@ sub edit_preference {
     return $self->_edit($self->host . EDIT_PREF_PATH, k => $key, v => $val);
 }
 
+
 sub opml {
     my ($self) = @_;
 
@@ -386,6 +396,7 @@ sub opml {
     return $res->decoded_content;
 }
 
+
 sub ping {
     my ($self) = @_;
     my $res = $self->_request(GET($self->host . PING_PATH)) or return;
@@ -396,6 +407,7 @@ sub ping {
     return;
 }
 
+
 ## Private interface
 
 sub _request {
@@ -408,12 +420,14 @@ sub _request {
         $req->uri->scheme('https');
     }
     elsif ('GET' eq $req->method and 'https' ne $req->uri->scheme) {
-        $req->uri->scheme($self->scheme);
+        $req->uri->scheme($self->_scheme);
     }
 
     $req->uri->query_param(ck => time * 1000);
     $req->uri->query_param(client => $self->ua->agent);
 
+    $req->header(AppId => $self->appid, AppKey => $self->appkey)
+        if defined $self->appid and defined $self->appkey;
     $req->header(authorization => 'GoogleLogin auth=' . $self->auth)
         if $self->auth;
 
@@ -451,6 +465,7 @@ sub _request {
     return $res;
 }
 
+
 sub _login {
     my ($self, $force) = @_;
 
@@ -477,6 +492,7 @@ sub _login {
     return 1;
 }
 
+
 sub _token {
     my ($self, $force) = @_;
 
@@ -491,10 +507,12 @@ sub _token {
     return $self->token($res->decoded_content);
 }
 
+
 sub _public {
     return ! ($_[0]->username and $_[0]->password);
 }
 
+
 sub _encode_type {
     my ($type, $val, $escape) = @_;
 
@@ -508,6 +526,7 @@ sub _encode_type {
     return wantarray ? @paths : shift @paths;
 }
 
+
 sub _encode_feed {
     my ($feed, $escape) = @_;
 
@@ -522,6 +541,7 @@ sub _encode_feed {
     return @paths;
 }
 
+
 sub _encode_tag {
     my ($tag) = @_;
 
@@ -537,6 +557,7 @@ sub _encode_tag {
     return @paths;
 }
 
+
 sub _encode_state {
     my ($state) = @_;
 
@@ -552,6 +573,7 @@ sub _encode_state {
     return @paths;
 }
 
+
 sub _encode_entry {
     my ($entry) = @_;
 
@@ -567,6 +589,7 @@ sub _encode_entry {
     return @paths;
 }
 
+
 sub _feed {
     my ($self, $type, $val, %params) = @_;
     return unless $val;
@@ -577,9 +600,6 @@ sub _feed {
     my $uri = URI->new($path . '/' . _encode_type($type, $val, 1));
 
     my %fields;
-    if (my $count = $params{count}) {
-        $fields{n} = $count;
-    }
     if (my $start_time = $params{start_time}) {
         $fields{ot} = $start_time;
     }
@@ -601,10 +621,11 @@ sub _feed {
 
     $uri->query_form(\%fields);
 
-    my $feed = (__PACKAGE__.'::Feed')->new(request => GET($uri), %params);
+    my $feed = (__PACKAGE__.'::Feed')->new(_request => GET($uri), %params);
     return $self->more($feed);
 }
 
+
 sub _list {
     my ($self, $url) = @_;
 
@@ -615,7 +636,7 @@ sub _list {
 
     my $res = $self->_request(GET($uri)) or return;
 
-    my $ref = eval { from_json($res->decoded_content) } or do {
+    my $ref = eval { JSON->new->decode($res->decoded_content) } or do {
         $self->error("Failed to parse JSON response: $@");
         return;
     };
@@ -630,6 +651,7 @@ sub _list {
     return @$aref
 }
 
+
 sub _edit {
     my ($self, $url, %fields) = @_;
     my $uri = URI->new($url);
@@ -643,6 +665,7 @@ sub _edit {
     return;
 }
 
+
 sub _edit_tag {
     my ($self, $type, $tag, %params) = @_;
     return unless $tag;
@@ -674,6 +697,7 @@ sub _edit_tag {
     return $self->_edit($url, %fields);
 }
 
+
 sub _states {
     return qw(
         read kept-unread fresh starred broadcast reading-list
@@ -682,6 +706,7 @@ sub _states {
     );
 }
 
+
 sub _listify {
     my ($aref) = @_;
     return (1 == @$aref and 'ARRAY' eq ref $aref->[0]) ? @$aref : $aref;
@@ -702,8 +727,10 @@ WebService::Google::Reader - Perl interface to the Google Reader API
 
     my $reader = WebService::Google::Reader->new(
         host     => 'www.inoreader.com',
-        username => $user,
-        password => $pass,
+        appid    => $appid,
+        appkey   => $appey,
+        username => $username,
+        password => $password,
     );
 
     my $feed = $reader->unread(count => 100);
@@ -740,6 +767,12 @@ The hostname of the service.
 Required for accessing any personalized or account-related functionality
 (reading-list, editing, etc.).
 
+=item B<appid> and B<appkey>
+
+Inoreader requires users to also obtain an appid and appkey.
+
+See L<http://www.inoreader.com/developers/register-app>
+
 =item B<https> / B<secure>
 
 Use https scheme for all requests, even when not required.
@@ -862,8 +895,8 @@ The sort order of the entries: B<desc> (default) or B<asc> in time.
 
 =item B<more> / B<previous> / B<next>
 
-A feed generator only returns B<$count> entries. If more are available,
-calling this method will return a feed with the next B<$count> entries.
+A feed generator only returns B<count> entries. If more are available,
+calling this method will return a feed with the next B<count> entries.
 
 =back
 
@@ -1113,27 +1146,35 @@ The following private methods may be of use to others.
 
 =item B<_login>
 
-This is automatically called from within methods that require
-authorization.  An optional parameter is accepted which when true, will
-force a login even if a previous login was successful. The end result of
-a successful login is to set the auth token.
+This is automatically called from within methods that require authorization.
+An optional parameter is accepted which when true, will force a login even if
+a previous login was successful. The end result of a successful login is to
+set the B<auth> token.
 
 =item B<_request>
 
-Given an C<HTTP::Request>, this will perform the request and if the
-response indicates a bad (expired) token, it will request another token
-before performing the request again. Returns an C<HTTP::Response> on
-success, false on failure (check B<error>).
-
-=item B<_token>
-
-This is automatically called from within methods that require a user token.
-If successful, the token is available via the B<token> accessor.
+Given an C<HTTP::Request>, this will perform the request and if the response
+indicates a bad (expired) token, it will request another token before
+performing the request again. Returns an C<HTTP::Response> on success, false
+on failure (check B<error>).
 
 =item B<_states>
 
 Returns a list of all the known states. See L</STATES>.
 
+=item B<_token>
+
+This is automatically called from within methods that require a user token. It
+fetches a new token when it becomes stale.
+
+=item B<auth>
+
+Accessor for the auth token, which is set after a successful B<_login>.
+
+=item B<token>
+
+Accessor for the user token, which is set by B<_token>.
+
 =back
 
 =head1 TAGS
@@ -1198,7 +1239,7 @@ L<https://groups.google.com/group/fougrapi/>
 
 L<http://code.google.com/p/pyrfeed/wiki/GoogleReaderAPI>
 
-L<http://wiki.inoreader.com/doku.php?id=api>
+L<http://www.inoreader.com/developers/>
 
 =head1 REQUESTS AND BUGS
 
@@ -1241,10 +1282,10 @@ L<http://search.cpan.org/dist/WebService-Google-Reader/>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2013 gray <gray at cpan.org>, all rights reserved.
+Copyright (C) 2007-2017 gray <gray at cpan.org>, all rights reserved.
 
-This library is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =head1 AUTHOR
 
