@@ -2,18 +2,26 @@ package WebService::Google::Reader::Feed;
 
 use strict;
 use warnings;
-use parent qw(XML::Atom::Feed Class::Accessor::Fast);
+use parent qw(XML::Atom::Feed);
 
 use WebService::Google::Reader::Constants qw(NS_GOOGLE_READER);
 
-__PACKAGE__->mk_accessors(qw(continuation count ids request));
+use Class::Tiny qw(continuation count _ids _request);
+
 
 sub new {
     my ($class, %params) = @_;
-    $params{count} ||= 20;
-    return bless \%params, $class;
+    my $self = bless \%params, $class;
+
+    $self->count(40) unless $self->count and 0 < $self->count;
+    if (my $req = $self->_request) {
+        $req->uri->query_param(n => $self->count);
+    }
+
+    return $self;
 }
 
+
 sub init {
     my $self = shift;
 
@@ -26,6 +34,7 @@ sub init {
     return $self;
 }
 
+
 # XML::Atom::Feed::entries() returns undef when there are no entries,
 # instead of an empty list, but only when using XML::LibXML.
 sub entries {
@@ -62,9 +71,13 @@ Subclass of C<XML::Atom::Feed>.
 
 Returns the continuation string, if any is present.
 
+=head2 count
+
+The number of entries per fetch: defaults to 40.
+
 =head2 entries
 
-Fixes bug in C<XML::Atom::Feed::entries()>.
+Override the method from C<XML::Atom::Feed> to work around a bug.
 
 =head1 SEE ALSO
 
